#-----------------------------------------------------------------------------
#
# Thomas Thomassen
# thomas[at]thomthom[dot]net
#
#-----------------------------------------------------------------------------

unless Object::RUBY_PLATFORM =~ /mswin32/i
	require 'Win32API'
	require 'C:/Ruby/lib/ruby/1.8/win32/registry.rb' if defined?(Win32::Registry).nil?
end

#-----------------------------------------------------------------------------

module Toolbars
	
	@cache_mac_names = nil
	
	# @return [UI::Toolbar, Toolbars::SketchupToolbar, nil] Toolbar object upon success, +nil+ upon error.
	def self.[](name)
		if UI.toolbar_names.include?(name)
			return Toolbars::SketchupToolbar.new(name)
		elsif self.ruby_names.to_a.include?(name)
			return UI.toolbar(name)
		else
			return nil
		end
	end
	
	# Creates an array of all accessible toolbar names.
	#
	# @example Show all toolbars.
	#   Toolbar.names.each { |name|
	#       t = Toolbars[name]
	#       t.show
	#   }
	#
	# @return [Array<UI::Toolbar, Toolbars::SketchupToolbar>, nil] Array of Toolbar objects.
	def self.names
		return UI.toolbar_names.concat(self.ruby_names.to_a)
	end
	
	# Creates an array of Ruby API created toolbar names. The names can be used to create a +Toolbar+
	# object refering to the existing toolbar.
	#
	# @example Show all Ruby toolbars.
	#   Toolbars.ruby_names.each { |name|
	#       t = UI::toolbar(name)
	#       t.show
	#   }
	#
	# *Note:* It is not possible to add items to the +Toolbar+ object you create. Only contol 
	# visibility.
	#
	# @return [Array<String>, nil] Array of names upon success, +nil+ upon error.
	def self.ruby_names
		return (self.is_mac?) ? self.toolbars_mac : self.toolbars_win
	end
	
	
	# Wrapper class to mimic Toolbar methods for native Sketchup toolbars.
	class SketchupToolbar
		
		attr(:name)
		
		def initialize(name)
			@name = name
		end
		
		def hide
			UI.set_toolbar_visible(@name, false)
		end
		
		def show
			UI.set_toolbar_visible(@name, true)
		end
		alias :restore :show
		
		def visible?
			UI.toolbar_visible?(@name)
		end
		
		def get_last_state
			return (visible?) ? 1 : 0
		end
		
	end # class
	
	def self.is_mac?
		return (Object::RUBY_PLATFORM =~ /darwin/i) ? true : false
	end
	
	private
	
	def self.toolbars_mac
		# In OSX 10.3 the plists where in XML format. In 10.4 and after
		# the plists are in binary format.
		#
		# Convertion tool: plutil -convert xml1 path/to/preferences.plist
		#
		# (!) Limitation: plist only list toolbars that's been shown before.
		# Can the list be obtained elsewhere?
		#
		# /Users/<user>/Library/Preferences/com.google.sketchupfree.plist
		# /Users/<user>/Library/Preferences/com.google.sketchupfree7.plist
		#
		# How preferences are stored:
		# http://developer.apple.com/mac/library/documentation/MacOSX/Conceptual/BPRuntimeConfig/Articles/UserPreferences.html#//apple_ref/doc/uid/20002092
		
		return @cache_mac_names unless @cache_mac_names.nil?
		
		# debug
		puts 'Building Toolbar List...'
		
		# Not sure what the best way to get the info from the plists are. But this works as a proof
		# of concept.
		#
		# Because it involves reading files and scanning with regex the result is cached
		# for later use.
		#
		# Not sure when the plists are written to. So it's unknown if it's written to as a toolbar
		# property changes. Run time changes will be lost.
		
		# Find the Bundle ID.
		# Look in the Info.plist file in the application package. This is in XML format.
		path = ENV['IG_ROOT'].match(/(\/.+\/)/)[1]
		begin
			file = File.new(path + 'Info.plist', 'r')
			data = file.read
			file.close
		rescue
			return nil
		end
		bundle_id = data.match(/<key>CFBundleIdentifier<\/key>\s*?<string\>(.*?)<\/string>/)[1]
		
		# Locate preferences plist file.
		path = ENV['HOME'] + '/Library/Preferences/'
		filename = bundle_id + '.plist'
		begin
			file = File.new(path + filename, 'r')
			data = file.read
			file.close
		rescue
			return nil
		end
		# This regex should work on XML and Binary plists.
		toolbars = data.scan(/SketchUp\.Toolbars\.(.*?)\.Visible/).flatten
		toolbars.delete('SketchUp') # This isn't a Ruby toolbar.
		
		# Cache the results
		@cache_mac_names = toolbars
		
		return toolbars
	end
	
	def self.toolbars_win
		# Determine sketchup version. Registry path will vary depending on this.
		# (i) This works for SU6 & 7. Untested for older versions.
		version = Sketchup.version.split('.')[0]
		toolbars = []
		begin
			# First get the count of toolbars stored in registry.
			path_summary = "Software\\Google\\SketchUp#{version}\\ToolbarsUser-Summary"
			Win32::Registry::HKEY_CURRENT_USER.open(path_summary) { |r_summary|
				bars = r_summary.read_i('Bars')
				# Try to open the registry section for each toolbar.
				for i in (0..bars-1)
					section = "ToolbarsUser-Bar#{i}"
					begin
						path_toolbar = "Software\\Google\\SketchUp#{version}\\#{section}"
						# Not all sections contain the name of a toolbar - so look for
						# the key 'BarName'.
						Win32::Registry::HKEY_CURRENT_USER.open(path_toolbar) { |reg|
							reg.each { |subkey, type, data|
								if type == Win32::Registry::REG_SZ && subkey == 'BarName'
									toolbars << data
									break
								end
							}
						}
					rescue
						next
					end
				end
			}
			return toolbars
		rescue
			# (?) Return empty array instead?
			return nil
		end

	end # def

end # module Toolbars